  clc;
  close all;

  format compact;
  format short g;

  global prob;
    
%% Initialize
  [prob.state.fem, prob.adjoint.fem] = getInitFem;
  prob.fem.dof = getDof;
  
%% Set up EIM problem
  prob.EIM.tol = 1e-5;
  prob.EIM.n_train = 2000;
	
%% Offline Stage: Perform empirical interpolation
  performEmpInt;

  figure(1)
  semilogy(1:length(prob.EIM.eps_star),prob.EIM.eps_star,'r-');
  title('EIM - Maximal Approximation Error during Greedy');
  xlabel Q; 
  ylabel('$\max\limits_{g \in G} \Vert g - I_Q g \Vert_\infty$', 'Interpreter', 'LaTex');
  legend('EIM Error');
  drawnow;

%% Pre-assemble calN-dependent systems for approximation I_Q g

  performPreAssembling_equ;

% test u(\mu)-u^Q(\mu)
  n_test = 10;
  Xi_test = -10.^(-2 + 2 .* rand(n_test,2));

  err_X = zeros(prob.EIM.Qmax,n_test);
  for n=1:n_test
    mu = num2cell(Xi_test(n,:));
    u = getSol('state',mu{:},0);
  
    for q=1:prob.EIM.Qmax
     % ===== TODO ========= 

     % uAppr = ...
     % err_X(q,n) = ....
     
     % ==================== 
    end
  end

  figure(2)
  semilogy(1:prob.EIM.Qmax,[min(err_X,[],2),mean(err_X,2),max(err_X,[],2)]);
  hold on;
  semilogy(1:prob.EIM.Qmax,prob.EIM.eps_star(2:prob.EIM.Qmax+1),'k:');
  xlabel('Q');
  ylabel('$||u(\mu)-u^Q(\mu)||_X$', 'Interpreter','LaTex');
  legend('min','mean','max');
  title('EIM: Approximation error for state u(\mu)');
  drawnow;


%% Train RB systems for both primal and dual problem

  n_train = 2000;
  Xi_train = -10.^(-2 + 2 .* rand(n_train,2));
  
  equ_str={'state','adjoint'};
  tol = [1e-4,1e-4];
  for i=1:2
    coll = ...
      struct('mu', cellfun(@(x) num2cell(x),num2cell(Xi_train,2)', ...
                           'UniformOutput',false), ...
             'theta',arrayfun(@(x) x,prob.(equ_str{i}).theta(Xi_train(:,1), ...
                              Xi_train(:,2),inf),'UniformOutput',false), ...
             'coercivity', 1, 'rb_coeffs', 0, 'R', 0, 'E', 0, 'errEst', 0);
	  trainRB_Greedy(coll,equ_str{i},'X',100,tol(i));
  end

%% Test solution of state equation

  n_test = 10;
  Xi_test = -10.^(-2 + 2 .* rand(2,n_test));
  
  sol = cell2mat(arrayfun(@(mu1,mu2) getSol('state', mu1,mu2,0), ...
                            Xi_test(1,:),Xi_test(2,:),'UniformOutput',false));
                        
  [Q,N] = meshgrid(1:prob.EIM.Qmax-1,1:prob.state.RB.Nmax);
  
  err_max = zeros(size(Q));
  eta_min = zeros(size(Q));
  eta_max = zeros(size(Q));

    
  for i=1: size(Q,1)
      for j=1:size(Q,2)
        err = zeros(n_test,1);
        errEst = zeros(n_test,1);

        for n=1:n_test
          theta = prob.state.theta(Xi_test(1,n),Xi_test(2,n),Q(i,j));
          rb_coeffs = getRBAppr(prob.state.RB,theta,N(i,j));
          errEst(n) = getErrEst_RB(prob.state.RB,theta,rb_coeffs,1);

          diff = sol(:,n)-prob.state.RB.Basis(:,1:N(i,j))*rb_coeffs;
          err(n) = sqrt(diff'*prob.fem.innProd.X*diff);
        end

        err_max(i,j) = max(err);
        eta_min(i,j) = min(errEst./err);        
        eta_max(i,j) = max(errEst./err);
      end
  end

   figure(3);
   surf(Q,N,err_max);
   set(gca, 'ZScale', 'log');
   xlabel Q;
   ylabel N;
   view(140,20);
   zlabel('$\max_\mu ||sol(\mu)-sol^{N,Q}(\mu)||_X$', 'Interpreter','LaTeX');
   title('Error of EIM-RB approximation of u');
      
   figure(4);
   surf(Q,N,eta_min);
   set(gca, 'ZScale', 'log');
   xlabel Q;
   ylabel N;
   view(210,20);
   zlabel('$\min_\mu \eta^{N,Q}(\mu)$', 'Interpreter','LaTeX');
   title('Minimum effectivity over test set');
  
   figure(5);
   surf(Q,N,eta_max);
   set(gca, 'ZScale', 'log');
   xlabel Q;
   ylabel N;
   view(210,20);
   zlabel('$\max_\mu \eta^{N,Q}(\mu)$', 'Interpreter','LaTeX');
   title('Maximum effectivity over test set');

%% Test output approximation

% Pre-assemble quantities for output approximation
	performPreAssembling_out;

  s = -sol'*prob.adjoint.fem.pre.rhs.L;

  [N_pr,N_du] = meshgrid(1:prob.state.RB.Nmax,0:prob.adjoint.RB.Nmax);
  
  err_max = zeros(size(N_pr));   
  for i=1: size(N_pr,1)
      for j=1:size(N_du,2)

        err = zeros(n_test,1);
        for n=1:n_test
		      thetaState = prob.state.theta(Xi_test(1,n),Xi_test(2,n),inf);
		      coeffsState_rb = getRBAppr(prob.state,thetaState,N_pr(i,j));

		      thetaAdjoint = prob.adjoint.theta(Xi_test(1,n),Xi_test(2,n),inf);
		      coeffsAdjoint_rb = getRBAppr(prob.adjoint,thetaAdjoint,N_du(i,j));

		      err(n) = abs(s(n)-getOutputAppr(coeffsState_rb,coeffsAdjoint_rb,thetaState));
        end

        err_max(i,j) = max(err);
      end
  end


  
  figure(6);
  surf(N_pr,N_du,err_max);
  set(gca, 'ZScale', 'log')
  xlabel N_{pr};
  ylabel N_{du};
  view(210,20);
  zlabel('$\max_\mu |s(\mu)-s^{N_{pr},N_{du}}(\mu)|$','Interpreter','LaTeX');
