function performEmpInt

  time = cputime;
  
  global prob;
  EIM = prob.EIM;
  
  fprintf(1,'Perform Empirical Interpolation (n_train=%d, tol=%.2e) ... ', ...
            EIM.n_train, EIM.tol);

  fem = prob.state.fem;

  Xi_train = -10.^(-2 + 2 .* rand(EIM.n_train,2));
  mu = num2cell(Xi_train,1);
  x = {fem.mesh.p(1,:)',fem.mesh.p(2,:)'};
  
% evaluate g for all values of mu in all points x
  X = cellfun(@(x) repmat(x,1,numel(mu{1})),x,'UniformOutput',false);
  Mu = cellfun(@(mu) repmat(mu',numel(x{1}),1),mu,'UniformOutput',false);
  G = g(X{:},Mu{:},0);
  
% apply actual empirical interpolation procedure  
  [EIM.Basis,S_ind,EIM.T_ind,EIM.Gbar,EIM.eps_star] = empIntAlg(G,EIM.tol);


% postprocessing
  EIM.Q_Qprime = 1; 
  EIM.Qmax = length(EIM.eps_star)-EIM.Q_Qprime;
  EIM.T = cellfun(@(x) x(EIM.T_ind),x,'UniformOutput',false);
  EIM.S = cellfun(@(mu) mu(S_ind),mu,'UniformOutput',false);  
  EIM.Xi_T = G(EIM.T_ind, S_ind);
  
  prob.EIM = EIM;
  
  fprintf(1,'done! CPU time: %.2fs.\n',cputime-time);