function trainRB_Greedy(coll,equ_str,Norm,Nmax,tol)

time = cputime;

global prob;

%% Initialize parameters and storage

  Nmax = eval('Nmax','100');
  tol = eval('tol','1e-5');
  n_train = numel(coll);
  Xi_train_ind = 1:n_train;
  S_ind = zeros(1,0);
  
    fprintf(1,'Perform greedy for %s (n_train=%d, tol=%.2e) ... ', ...
      equ_str,n_train,tol);

% store norm to be used  
  RB.Norm = Norm;
  
% initialize RB matrices
  %RB.pre.lhs.K = cell(1,prob.EIM.Qmax+1);
  RB.pre.lhs.K = [];
  RB.pre.rhs.L = [];
  RB.pre.lhs.Qmax = numel(prob.state.fem.pre.lhs.K);
  RB.pre.rhs.Qmax = size(prob.state.fem.pre.rhs.L,2);

  
% initialize S, Basis and Riesz Representors
  RB.Basis = zeros(prob.fem.dof.num,0);
  RB.RieszReprs = zeros(length(prob.fem.dof.ind),0);
  RB.innProdRieszReprs = [];
  RB.Greedy_errors = [];
  RB.Nmax = 0;

  % compute Riesz representors 
  innProd = prob.fem.innProd.(RB.Norm);
  innProd = innProd(prob.fem.dof.ind,prob.fem.dof.ind);
% ... for rhs 
    RB.RieszReprs = ...
      innProd\prob.(equ_str).fem.pre.rhs.L(prob.fem.dof.ind,:);
    RB.innProdRieszReprs = RB.RieszReprs'*innProd*RB.RieszReprs;
    
%% Define addBasisFun
% add orthonormalized u(mu) to RB basis
function addBasisFun

% compute solution
  u = getSol(equ_str,coll(errEst_max_ind).mu{:});
  
  N = size(RB.Basis,2);

% orthonormalize 
  if (N==0)
    r = u;
  else
    r = u - RB.Basis*(RB.Basis'*prob.fem.innProd.(RB.Norm)*u);
  end
  RB.Basis(:,end+1) = r/sqrt(r'*prob.fem.innProd.(RB.Norm)*r);
  
% add to S_ind and remove from 'Xi_train'
  S_ind(1,end+1) = errEst_max_ind;
  Xi_train_ind = setdiff(Xi_train_ind,errEst_max_ind);

% compute new pre-assembled RB quantities
  % lhs
  Qmax = prob.(equ_str).fem.pre.lhs.Qmax;
  N = size(RB.Basis,2);
  RB.pre.lhs.K = zeros(N^2, Qmax);
  for q=1:Qmax  
    K = RB.Basis' * prob.(equ_str).fem.pre.lhs.K{q}*RB.Basis;
    RB.pre.lhs.K(:,q) = K(:);
  end
  RB.pre.lhs.Qmax = numel(prob.(equ_str).fem.pre.lhs.K);
  % rhs
  Qmax = prob.(equ_str).fem.pre.rhs.Qmax;
  RB.pre.rhs.L = RB.Basis'*prob.(equ_str).fem.pre.rhs.L(:,1:Qmax);
  RB.pre.rhs.Qmax = size(prob.(equ_str).fem.pre.rhs.L,2);


% compute Riesz representors
  innProd = prob.fem.innProd.(RB.Norm);
  innProd = innProd(prob.fem.dof.ind,prob.fem.dof.ind);
% ... for lhs
  Kn = cell2mat(cellfun(@(K) -K*RB.Basis(:,N), ...
                        prob.(equ_str).fem.pre.lhs.K, ...
                        'UniformOutput',false));
  RB.RieszReprs = [RB.RieszReprs innProd\Kn(prob.fem.dof.ind,:)];
% compute inner product of duals
  RB.innProdRieszReprs = RB.RieszReprs'*innProd*RB.RieszReprs;

end
  
%% Training
  
% start greedy
  while (size(RB.Basis,2)<Nmax)

  % compute error estimator for all remaining values in Xi_train
    for i=Xi_train_ind
    % compute RB approximation
      coll(i).rb_coeffs = getRBAppr(RB,coll(i),size(RB.Basis,2));
    % compute error estimator
      [coll(i).errEst,coll(i).R,coll(i).E] = ...
        getErrEst_RB(RB,coll(i));
    end
    errEst = [coll.errEst];
    errEst(S_ind) = 0;

  % determine index of argmax in Xi_train
    [errEst_max,errEst_max_ind] = max(errEst);
    
  % store Greedy errors
    RB.Greedy_errors(end+1) = errEst_max;

  % break, if maximum error estimator is less than tol
    if (errEst_max<tol)
      break;
    end
    
  % add solution for identified parameter to basis
    addBasisFun;
    
    RB.Nmax = size(RB.Basis,2);
    
    [RB.Nmax RB.Greedy_errors(end)]
    
  end
  
% store parameters
  RB.S = {coll(S_ind).mu};
  prob.(equ_str).RB = RB;


  fprintf(1,'done! CPU time: %.2fs.\n',cputime-time);


end    